@@ -41,6 +41,7 @@ const SUPPORTED_WASM_FEATURES: &[(&str, &str)] = &[
     ("reference-types", "enables support for reference types"),
     ("simd", "enables support for proposed SIMD instructions"),
     ("threads", "enables support for WebAssembly threads"),
+    ("memory64", "enables support for 64-bit memories"),
 ];
 
 const SUPPORTED_WASI_MODULES: &[(&str, &str)] = &[
@@ -93,17 +94,13 @@ lazy_static::lazy_static! {
 }
 
 pub mod commands;
-mod obj;
 
 use anyhow::{bail, Result};
 use std::collections::HashMap;
 use std::path::PathBuf;
 use structopt::StructOpt;
-use target_lexicon::Triple;
 use wasmtime::{Config, ProfilingStrategy, Strategy};
 
-pub use obj::compile_to_obj;
-
 fn pick_compilation_strategy(cranelift: bool, lightbeam: bool) -> Result<Strategy> {
     Ok(match (lightbeam, cranelift) {
         (true, false) => Strategy::Lightbeam,
@@ -297,7 +294,10 @@ impl CommonOptions {
     fn config(&self, target: Option<&str>) -> Result<Config> {
         let mut config = Config::new();
 
-        // Set the target before setting any cranelift options
+        // Set the compiler and target before setting any cranelift options,
+        // since the strategy determines which compiler is in use and the target
+        // will reset any target-specific options.
+        config.strategy(pick_compilation_strategy(self.cranelift, self.lightbeam)?)?;
         if let Some(target) = target {
             config.target(target)?;
         }
@@ -306,7 +306,6 @@ impl CommonOptions {
             .cranelift_debug_verifier(self.enable_cranelift_debug_verifier)
             .debug_info(self.debug_info)
             .cranelift_opt_level(self.opt_level())
-            .strategy(pick_compilation_strategy(self.cranelift, self.lightbeam)?)?
             .profiler(pick_profiling_strategy(self.jitdump, self.vtune)?)?
             .cranelift_nan_canonicalization(self.enable_cranelift_nan_canonicalization);
 
@@ -362,6 +361,7 @@ impl CommonOptions {
             .wasm_multi_value(features.multi_value || self.enable_multi_value || self.enable_all)
             .wasm_threads(features.threads || self.enable_threads || self.enable_all)
             .wasm_multi_memory(features.multi_memory || self.enable_multi_memory || self.enable_all)
+            .wasm_memory64(features.memory64 || self.enable_all)
             .wasm_module_linking(
                 features.module_linking || self.enable_module_linking || self.enable_all,
             );
@@ -437,7 +437,7 @@ fn parse_wasm_features(features: &str) -> Result<wasmparser::WasmFeatures> {
         deterministic_only: false,
         multi_memory: all.unwrap_or(values["multi-memory"].unwrap_or(false)),
         exceptions: false,
-        memory64: false,
+        memory64: all.unwrap_or(values["memory64"].unwrap_or(false)),
     })
 }
 
@@ -524,11 +524,6 @@ fn parse_cranelift_flag(name_and_value: &str) -> Result<(String, String)> {
     Ok((name, value))
 }
 
-fn parse_target(s: &str) -> Result<Triple> {
-    use std::str::FromStr;
-    Triple::from_str(&s).map_err(|e| anyhow::anyhow!(e))
-}
-
 #[cfg(test)]
 mod test {
     use super::*;
@@ -561,7 +556,7 @@ mod test {
         assert!(!deterministic_only); // Not supported
         assert!(multi_memory);
         assert!(!exceptions); // Not supported
-        assert!(!memory64); // Not supported
+        assert!(memory64);
 
         Ok(())
     }
@@ -603,7 +598,7 @@ mod test {
     fn test_multiple_features() -> Result<()> {
         let options = CommonOptions::from_iter_safe(vec![
             "foo",
-            "--wasm-features=-reference-types,simd,multi-memory",
+            "--wasm-features=-reference-types,simd,multi-memory,memory64",
         ])?;
 
         let wasmparser::WasmFeatures {
@@ -630,7 +625,7 @@ mod test {
         assert!(!deterministic_only); // Not supported
         assert!(multi_memory);
         assert!(!exceptions); // Not supported
-        assert!(!memory64); // Not supported
+        assert!(memory64);
 
         Ok(())
     }
@@ -675,6 +670,7 @@ mod test {
     feature_test!(test_simd_feature, simd, "simd");
     feature_test!(test_threads_feature, threads, "threads");
     feature_test!(test_multi_memory_feature, multi_memory, "multi-memory");
+    feature_test!(test_memory64_feature, memory64, "memory64");
 
     #[test]
     fn test_default_modules() {
