@@ -392,7 +392,7 @@ fn iloop_with_fuel() {
 fn fuel_eventually_finishes() {
     let engine = Engine::new(Config::new().async_support(true).consume_fuel(true)).unwrap();
     let mut store = Store::new(&engine, ());
-    store.out_of_fuel_async_yield(u32::max_value(), 10);
+    store.out_of_fuel_async_yield(u64::max_value(), 10);
     let module = Module::new(
         &engine,
         "
@@ -428,11 +428,11 @@ fn async_with_pooling_stacks() {
             table_elements: 0,
             ..Default::default()
         },
-        instance_limits: InstanceLimits {
-            count: 1,
-            memory_reservation_size: 1,
-        },
+        instance_limits: InstanceLimits { count: 1 },
     });
+    config.dynamic_memory_guard_size(0);
+    config.static_memory_guard_size(0);
+    config.static_memory_maximum_size(65536);
 
     let engine = Engine::new(&config).unwrap();
     let mut store = Store::new(&engine, ());
@@ -457,11 +457,11 @@ fn async_host_func_with_pooling_stacks() -> Result<()> {
             table_elements: 0,
             ..Default::default()
         },
-        instance_limits: InstanceLimits {
-            count: 1,
-            memory_reservation_size: 1,
-        },
+        instance_limits: InstanceLimits { count: 1 },
     });
+    config.dynamic_memory_guard_size(0);
+    config.static_memory_guard_size(0);
+    config.static_memory_maximum_size(65536);
 
     let mut store = Store::new(&Engine::new(&config)?, ());
     let mut linker = Linker::new(store.engine());
@@ -610,3 +610,114 @@ fn resume_separate_thread3() {
     });
     assert!(f.call(&mut store, &[]).is_err());
 }
+
+#[test]
+fn recursive_async() -> Result<()> {
+    let mut store = async_store();
+    let m = Module::new(
+        store.engine(),
+        "(module
+            (func (export \"overflow\") call 0)
+            (func (export \"normal\"))
+        )",
+    )?;
+    let i = run(Instance::new_async(&mut store, &m, &[]))?;
+    let overflow = i.get_typed_func::<(), (), _>(&mut store, "overflow")?;
+    let normal = i.get_typed_func::<(), (), _>(&mut store, "normal")?;
+    let f2 = Func::wrap0_async(&mut store, move |mut caller| {
+        Box::new(async move {
+            // recursive async calls shouldn't immediately stack overflow...
+            normal.call_async(&mut caller, ()).await?;
+
+            // ... but calls that actually stack overflow should indeed stack
+            // overflow
+            let err = overflow.call_async(&mut caller, ()).await.unwrap_err();
+            assert_eq!(err.trap_code(), Some(TrapCode::StackOverflow));
+            Ok(())
+        })
+    });
+    run(f2.call_async(&mut store, &[]))?;
+    Ok(())
+}
+
+#[test]
+fn linker_module_command() -> Result<()> {
+    run(async {
+        let mut store = async_store();
+        let mut linker = Linker::new(store.engine());
+        let module1 = Module::new(
+            store.engine(),
+            r#"
+                (module
+                    (global $g (mut i32) (i32.const 0))
+
+                    (func (export "_start"))
+
+                    (func (export "g") (result i32)
+                        global.get $g
+                        i32.const 1
+                        global.set $g)
+                )
+            "#,
+        )?;
+        let module2 = Module::new(
+            store.engine(),
+            r#"
+                (module
+                    (import "" "g" (func (result i32)))
+
+                    (func (export "get") (result i32)
+                        call 0)
+                )
+            "#,
+        )?;
+
+        linker.module_async(&mut store, "", &module1).await?;
+        let instance = linker.instantiate_async(&mut store, &module2).await?;
+        let f = instance.get_typed_func::<(), i32, _>(&mut store, "get")?;
+        assert_eq!(f.call_async(&mut store, ()).await?, 0);
+        assert_eq!(f.call_async(&mut store, ()).await?, 0);
+
+        Ok(())
+    })
+}
+
+#[test]
+fn linker_module_reactor() -> Result<()> {
+    run(async {
+        let mut store = async_store();
+        let mut linker = Linker::new(store.engine());
+        let module1 = Module::new(
+            store.engine(),
+            r#"
+                (module
+                    (global $g (mut i32) (i32.const 0))
+
+                    (func (export "g") (result i32)
+                        global.get $g
+                        i32.const 1
+                        global.set $g)
+                )
+            "#,
+        )?;
+        let module2 = Module::new(
+            store.engine(),
+            r#"
+                (module
+                    (import "" "g" (func (result i32)))
+
+                    (func (export "get") (result i32)
+                        call 0)
+                )
+            "#,
+        )?;
+
+        linker.module_async(&mut store, "", &module1).await?;
+        let instance = linker.instantiate_async(&mut store, &module2).await?;
+        let f = instance.get_typed_func::<(), i32, _>(&mut store, "get")?;
+        assert_eq!(f.call_async(&mut store, ()).await?, 0);
+        assert_eq!(f.call_async(&mut store, ()).await?, 1);
+
+        Ok(())
+    })
+}
