@@ -455,9 +455,6 @@ fn func_write_nothing() -> anyhow::Result<()> {
 }
 
 #[test]
-// Note: Cranelift only supports refrerence types (used in the wasm in this
-// test) on x64.
-#[cfg(target_arch = "x86_64")]
 fn return_cross_store_value() -> anyhow::Result<()> {
     let wasm = wat::parse_str(
         r#"
@@ -490,9 +487,6 @@ fn return_cross_store_value() -> anyhow::Result<()> {
 }
 
 #[test]
-// Note: Cranelift only supports refrerence types (used in the wasm in this
-// test) on x64.
-#[cfg(target_arch = "x86_64")]
 fn pass_cross_store_arg() -> anyhow::Result<()> {
     let mut config = Config::new();
     config.wasm_reference_types(true);
@@ -520,6 +514,7 @@ fn pass_cross_store_arg() -> anyhow::Result<()> {
 }
 
 #[test]
+#[cfg_attr(feature = "old-x86-backend", ignore)]
 fn externref_signature_no_reference_types() -> anyhow::Result<()> {
     let mut config = Config::new();
     config.wasm_reference_types(false);
@@ -884,3 +879,45 @@ fn wasm_ty_roundtrip() -> Result<(), anyhow::Error> {
     foo.call(&mut store, (-1, 1, 2.0, -3, 3, 4.0))?;
     Ok(())
 }
+
+#[test]
+fn typed_funcs_count_params_correctly_in_error_messages() -> anyhow::Result<()> {
+    let mut store = Store::<()>::default();
+    let module = Module::new(
+        store.engine(),
+        r#"
+            (module
+                (func (export "f") (param i32 i32))
+            )
+
+        "#,
+    )?;
+    let instance = Instance::new(&mut store, &module, &[])?;
+
+    // Too few parameters.
+    match instance.get_typed_func::<(), (), _>(&mut store, "f") {
+        Ok(_) => panic!("should be wrong signature"),
+        Err(e) => {
+            let msg = format!("{:?}", e);
+            assert!(dbg!(msg).contains("expected 0 types, found 2"))
+        }
+    }
+    match instance.get_typed_func::<(i32,), (), _>(&mut store, "f") {
+        Ok(_) => panic!("should be wrong signature"),
+        Err(e) => {
+            let msg = format!("{:?}", e);
+            assert!(dbg!(msg).contains("expected 1 types, found 2"))
+        }
+    }
+
+    // Too many parameters.
+    match instance.get_typed_func::<(i32, i32, i32), (), _>(&mut store, "f") {
+        Ok(_) => panic!("should be wrong signature"),
+        Err(e) => {
+            let msg = format!("{:?}", e);
+            assert!(dbg!(msg).contains("expected 3 types, found 2"))
+        }
+    }
+
+    Ok(())
+}
