@@ -1,6 +1,8 @@
 use anyhow::Result;
 use wasmtime::*;
 
+const WASM_PAGE_SIZE: usize = wasmtime_environ::WASM_PAGE_SIZE as usize;
+
 #[test]
 fn test_limits() -> Result<()> {
     let engine = Engine::default();
@@ -12,7 +14,7 @@ fn test_limits() -> Result<()> {
     let mut store = Store::new(
         &engine,
         StoreLimitsBuilder::new()
-            .memory_pages(10)
+            .memory_size(10 * WASM_PAGE_SIZE)
             .table_elements(5)
             .build(),
     );
@@ -23,7 +25,7 @@ fn test_limits() -> Result<()> {
     // Test instance exports and host objects hitting the limit
     for memory in std::array::IntoIter::new([
         instance.get_memory(&mut store, "m").unwrap(),
-        Memory::new(&mut store, MemoryType::new(Limits::new(0, None)))?,
+        Memory::new(&mut store, MemoryType::new(0, None))?,
     ]) {
         memory.grow(&mut store, 3)?;
         memory.grow(&mut store, 5)?;
@@ -43,7 +45,7 @@ fn test_limits() -> Result<()> {
         instance.get_table(&mut store, "t").unwrap(),
         Table::new(
             &mut store,
-            TableType::new(ValType::FuncRef, Limits::new(0, None)),
+            TableType::new(ValType::FuncRef, 0, None),
             Val::FuncRef(None),
         )?,
     ]) {
@@ -71,7 +73,12 @@ fn test_limits_memory_only() -> Result<()> {
         r#"(module (memory (export "m") 0) (table (export "t") 0 anyfunc))"#,
     )?;
 
-    let mut store = Store::new(&engine, StoreLimitsBuilder::new().memory_pages(10).build());
+    let mut store = Store::new(
+        &engine,
+        StoreLimitsBuilder::new()
+            .memory_size(10 * WASM_PAGE_SIZE)
+            .build(),
+    );
     store.limiter(|s| s as &mut dyn ResourceLimiter);
 
     let instance = Instance::new(&mut store, &module, &[])?;
@@ -79,7 +86,7 @@ fn test_limits_memory_only() -> Result<()> {
     // Test instance exports and host objects hitting the limit
     for memory in std::array::IntoIter::new([
         instance.get_memory(&mut store, "m").unwrap(),
-        Memory::new(&mut store, MemoryType::new(Limits::new(0, None)))?,
+        Memory::new(&mut store, MemoryType::new(0, None))?,
     ]) {
         memory.grow(&mut store, 3)?;
         memory.grow(&mut store, 5)?;
@@ -99,7 +106,7 @@ fn test_limits_memory_only() -> Result<()> {
         instance.get_table(&mut store, "t").unwrap(),
         Table::new(
             &mut store,
-            TableType::new(ValType::FuncRef, Limits::new(0, None)),
+            TableType::new(ValType::FuncRef, 0, None),
             Val::FuncRef(None),
         )?,
     ]) {
@@ -117,7 +124,12 @@ fn test_initial_memory_limits_exceeded() -> Result<()> {
     let engine = Engine::default();
     let module = Module::new(&engine, r#"(module (memory (export "m") 11))"#)?;
 
-    let mut store = Store::new(&engine, StoreLimitsBuilder::new().memory_pages(10).build());
+    let mut store = Store::new(
+        &engine,
+        StoreLimitsBuilder::new()
+            .memory_size(10 * WASM_PAGE_SIZE)
+            .build(),
+    );
     store.limiter(|s| s as &mut dyn ResourceLimiter);
 
     match Instance::new(&mut store, &module, &[]) {
@@ -128,7 +140,7 @@ fn test_initial_memory_limits_exceeded() -> Result<()> {
         ),
     }
 
-    match Memory::new(&mut store, MemoryType::new(Limits::new(25, None))) {
+    match Memory::new(&mut store, MemoryType::new(25, None)) {
         Ok(_) => unreachable!(),
         Err(e) => assert_eq!(
             e.to_string(),
@@ -155,7 +167,7 @@ fn test_limits_table_only() -> Result<()> {
     // Test instance exports and host objects *not* hitting the limit
     for memory in std::array::IntoIter::new([
         instance.get_memory(&mut store, "m").unwrap(),
-        Memory::new(&mut store, MemoryType::new(Limits::new(0, None)))?,
+        Memory::new(&mut store, MemoryType::new(0, None))?,
     ]) {
         memory.grow(&mut store, 3)?;
         memory.grow(&mut store, 5)?;
@@ -168,7 +180,7 @@ fn test_limits_table_only() -> Result<()> {
         instance.get_table(&mut store, "t").unwrap(),
         Table::new(
             &mut store,
-            TableType::new(ValType::FuncRef, Limits::new(0, None)),
+            TableType::new(ValType::FuncRef, 0, None),
             Val::FuncRef(None),
         )?,
     ]) {
@@ -206,7 +218,7 @@ fn test_initial_table_limits_exceeded() -> Result<()> {
 
     match Table::new(
         &mut store,
-        TableType::new(ValType::FuncRef, Limits::new(99, None)),
+        TableType::new(ValType::FuncRef, 99, None),
         Val::FuncRef(None),
     ) {
         Ok(_) => unreachable!(),
@@ -241,7 +253,12 @@ fn test_pooling_allocator_initial_limits_exceeded() -> Result<()> {
         r#"(module (memory (export "m1") 2) (memory (export "m2") 5))"#,
     )?;
 
-    let mut store = Store::new(&engine, StoreLimitsBuilder::new().memory_pages(3).build());
+    let mut store = Store::new(
+        &engine,
+        StoreLimitsBuilder::new()
+            .memory_size(3 * WASM_PAGE_SIZE)
+            .build(),
+    );
     store.limiter(|s| s as &mut dyn ResourceLimiter);
 
     match Instance::new(&mut store, &module, &[]) {
@@ -268,15 +285,12 @@ struct MemoryContext {
 }
 
 impl ResourceLimiter for MemoryContext {
-    fn memory_growing(&mut self, current: u32, desired: u32, maximum: Option<u32>) -> bool {
+    fn memory_growing(&mut self, current: usize, desired: usize, maximum: Option<usize>) -> bool {
         // Check if the desired exceeds a maximum (either from Wasm or from the host)
-        if desired > maximum.unwrap_or(u32::MAX) {
-            self.limit_exceeded = true;
-            return false;
-        }
+        assert!(desired < maximum.unwrap_or(usize::MAX));
 
-        assert_eq!(current as usize * 0x10000, self.wasm_memory_used,);
-        let desired = desired as usize * 0x10000;
+        assert_eq!(current as usize, self.wasm_memory_used);
+        let desired = desired as usize;
 
         if desired + self.host_memory_used > self.memory_limit {
             self.limit_exceeded = true;
@@ -371,3 +385,170 @@ fn test_custom_limiter() -> Result<()> {
 
     Ok(())
 }
+
+#[derive(Default)]
+struct MemoryGrowFailureDetector {
+    /// Arguments of most recent call to memory_growing
+    current: usize,
+    desired: usize,
+    /// Display impl of most recent call to memory_grow_failed
+    error: Option<String>,
+}
+
+impl ResourceLimiter for MemoryGrowFailureDetector {
+    fn memory_growing(&mut self, current: usize, desired: usize, _maximum: Option<usize>) -> bool {
+        self.current = current;
+        self.desired = desired;
+        true
+    }
+
+    fn memory_grow_failed(&mut self, err: &anyhow::Error) {
+        self.error = Some(err.to_string());
+    }
+
+    fn table_growing(&mut self, _current: u32, _desired: u32, _maximum: Option<u32>) -> bool {
+        true
+    }
+}
+
+#[test]
+fn custom_limiter_detect_grow_failure() -> Result<()> {
+    if std::env::var("WASMTIME_TEST_NO_HOG_MEMORY").is_ok() {
+        return Ok(());
+    }
+    let mut config = Config::new();
+    config.allocation_strategy(InstanceAllocationStrategy::Pooling {
+        strategy: PoolingAllocationStrategy::NextAvailable,
+        module_limits: ModuleLimits {
+            memory_pages: 10,
+            ..Default::default()
+        },
+        instance_limits: InstanceLimits {
+            ..Default::default()
+        },
+    });
+    let engine = Engine::new(&config).unwrap();
+    let linker = Linker::new(&engine);
+
+    let module = Module::new(&engine, r#"(module (memory (export "m") 0))"#)?;
+
+    let context = MemoryGrowFailureDetector::default();
+
+    let mut store = Store::new(&engine, context);
+    store.limiter(|s| s as &mut dyn ResourceLimiter);
+    let instance = linker.instantiate(&mut store, &module)?;
+    let memory = instance.get_memory(&mut store, "m").unwrap();
+
+    // Grow the memory by 640 KiB (10 pages)
+    memory.grow(&mut store, 10)?;
+
+    assert!(store.data().error.is_none());
+    assert_eq!(store.data().current, 0);
+    assert_eq!(store.data().desired, 10 * 64 * 1024);
+
+    // Grow past the static limit set by ModuleLimits.
+    // The ResourcLimiter will permit this, but the grow will fail.
+    assert_eq!(
+        memory.grow(&mut store, 1).unwrap_err().to_string(),
+        "failed to grow memory by `1`"
+    );
+
+    assert_eq!(store.data().current, 10 * 64 * 1024);
+    assert_eq!(store.data().desired, 11 * 64 * 1024);
+    assert_eq!(
+        store.data().error.as_ref().unwrap(),
+        "Memory maximum size exceeded"
+    );
+
+    drop(store);
+
+    Ok(())
+}
+
+// This test only works on Linux. It may be portable to MacOS as well,
+// but the original author did not have a machine available to test it.
+#[cfg(target_os = "linux")]
+#[test]
+fn custom_limiter_detect_os_oom_failure() -> Result<()> {
+    if std::env::var("WASMTIME_TEST_NO_HOG_MEMORY").is_ok() {
+        return Ok(());
+    }
+
+    let pid = unsafe { libc::fork() };
+    if pid == 0 {
+        // Child process
+        let r = std::panic::catch_unwind(|| {
+            // Ask Linux to limit this process to 128MiB of memory
+            let process_max_memory: usize = 128 * 1024 * 1024;
+            unsafe {
+                // limit process to 128MiB memory
+                let rlimit = libc::rlimit {
+                    rlim_cur: 0,
+                    rlim_max: process_max_memory as u64,
+                };
+                let res = libc::setrlimit(libc::RLIMIT_DATA, &rlimit);
+                assert_eq!(res, 0, "setrlimit failed: {}", res);
+            };
+
+            // Default behavior of on-demand memory allocation so that a
+            // memory grow will hit Linux for a larger mmap.
+            let engine = Engine::default();
+            let linker = Linker::new(&engine);
+            let module = Module::new(&engine, r#"(module (memory (export "m") 0))"#).unwrap();
+
+            let context = MemoryGrowFailureDetector::default();
+
+            let mut store = Store::new(&engine, context);
+            store.limiter(|s| s as &mut dyn ResourceLimiter);
+            let instance = linker.instantiate(&mut store, &module).unwrap();
+            let memory = instance.get_memory(&mut store, "m").unwrap();
+
+            // Small (640KiB) grow should succeed
+            memory.grow(&mut store, 10).unwrap();
+            assert!(store.data().error.is_none());
+            assert_eq!(store.data().current, 0);
+            assert_eq!(store.data().desired, 10 * 64 * 1024);
+
+            // Try to grow past the process's memory limit.
+            // This should fail.
+            let pages_exceeding_limit = process_max_memory / (64 * 1024);
+            let err_msg = memory
+                .grow(&mut store, pages_exceeding_limit as u64)
+                .unwrap_err()
+                .to_string();
+            assert!(
+                err_msg.starts_with("failed to grow memory"),
+                "unexpected error: {}",
+                err_msg
+            );
+
+            assert_eq!(store.data().current, 10 * 64 * 1024);
+            assert_eq!(
+                store.data().desired,
+                (pages_exceeding_limit + 10) * 64 * 1024
+            );
+            // The memory_grow_failed hook should show Linux gave OOM:
+            let err_msg = store.data().error.as_ref().unwrap();
+            assert!(
+                err_msg.starts_with("System call failed: Cannot allocate memory"),
+                "unexpected error: {}",
+                err_msg
+            );
+        });
+        // on assertion failure, exit 1 so parent process can fail test.
+        std::process::exit(if r.is_err() { 1 } else { 0 });
+    } else {
+        // Parent process
+        let mut wstatus: libc::c_int = 0;
+        unsafe { libc::wait(&mut wstatus) };
+        if libc::WIFEXITED(wstatus) {
+            if libc::WEXITSTATUS(wstatus) == 0 {
+                Ok(())
+            } else {
+                anyhow::bail!("child exited with failure");
+            }
+        } else {
+            anyhow::bail!("child didnt exit??")
+        }
+    }
+}
