@@ -1,11 +1,14 @@
 mod async_functions;
+mod call_hook;
 mod cli_tests;
 mod custom_signal_handler;
 mod debug;
 mod externals;
 mod fuel;
 mod func;
+mod funcref;
 mod fuzzing;
+mod gc;
 mod globals;
 mod host_funcs;
 mod iloop;
@@ -15,27 +18,21 @@ mod instance;
 mod invoke_func_via_table;
 mod limits;
 mod linker;
+mod memory;
 mod memory_creator;
 mod module;
 mod module_linking;
 mod module_serialize;
 mod name;
-mod native_hooks;
 mod pooling_allocator;
+mod relocs;
 mod stack_overflow;
 mod store;
 mod table;
 mod traps;
 mod wast;
 
-// TODO(#1886): Cranelift only supports reference types on x64.
-#[cfg(target_arch = "x86_64")]
-mod funcref;
-#[cfg(target_arch = "x86_64")]
-mod gc;
-
 /// A helper to compile a module in a new store with reference types enabled.
-#[cfg(target_arch = "x86_64")]
 pub(crate) fn ref_types_module(
     source: &str,
 ) -> anyhow::Result<(wasmtime::Store<()>, wasmtime::Module)> {
@@ -53,3 +50,11 @@ pub(crate) fn ref_types_module(
 
     Ok((store, module))
 }
+
+/// A helper determining whether the pooling allocator tests should be skipped.
+pub(crate) fn skip_pooling_allocator_tests() -> bool {
+    // There are a couple of issues when running the pooling allocator tests under QEMU:
+    // - high memory usage that may exceed the limits imposed by the environment (e.g. CI)
+    // - https://github.com/bytecodealliance/wasmtime/pull/2518#issuecomment-747280133
+    std::env::var("WASMTIME_TEST_NO_HOG_MEMORY").is_ok()
+}
